@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,32 +23,30 @@
 Test case for module Windows
 """
 
-from __future__ import absolute_import, print_function
-
 from tests import common
 
 from bleachbit.FileUtilities import extended_path, extended_path_undo
 from bleachbit.Windows import *
 from bleachbit import logger
 
-import functools
 import os
 import platform
 import shutil
 import sys
 import tempfile
 import unittest
+import mock
 from decimal import Decimal
 
 if 'win32' == sys.platform:
-    import _winreg
+    import winreg
     from win32com.shell import shell
 
 
 def put_files_into_recycle_bin():
     """Put a file and a folder into the recycle bin"""
     # make a file and move it to the recycle bin
-    tests = ('regular', u'unicode-emdash-u\u2014', 'long' + 'x' * 100)
+    tests = ('regular', 'unicode-emdash-u\u2014', 'long' + 'x' * 100)
     for test in tests:
         (fd, filename) = tempfile.mkstemp(
             prefix='bleachbit-recycle-file', suffix=test)
@@ -60,7 +58,7 @@ def put_files_into_recycle_bin():
     move_to_recycle_bin(dirname)
 
 
-@unittest.skipUnless('win32' == sys.platform, 'not running on windows')
+@common.skipUnlessWindows
 class WindowsTestCase(common.BleachbitTestCase):
 
     """Test case for module Windows"""
@@ -73,8 +71,10 @@ class WindowsTestCase(common.BleachbitTestCase):
         """Unit test for get_recycle_bin"""
         for f in get_recycle_bin():
             self.assertExists(extended_path(f))
-        if not common.destructive_tests('get_recycle_bin'):
-            return
+
+    @common.skipUnlessDestructive
+    def test_get_recycle_bin_destructive(self):
+        """Unit test the destructive part of get_recycle_bin"""
         put_files_into_recycle_bin()
         # clear recycle bin
         counter = 0
@@ -83,7 +83,7 @@ class WindowsTestCase(common.BleachbitTestCase):
             FileUtilities.delete(f)
         self.assertGreaterEqual(counter, 3, 'deleted %d' % counter)
         # now it should be empty
-        for f in get_recycle_bin():
+        for _f in get_recycle_bin():
             self.fail('recycle bin should be empty, but it is not')
 
     def _test_link_helper(self, mklink_option, clear_recycle_bin):
@@ -91,7 +91,7 @@ class WindowsTestCase(common.BleachbitTestCase):
         get_recycle_bin()
 
         It gets called four times for the combinations of the two
-        parameters. It's called by four unit tests four accounting
+        parameters. It's called by four unit tests for accounting
         purposes. In other words, we don't want to count a test as
         skipped if part of it succeeded.
 
@@ -110,7 +110,8 @@ class WindowsTestCase(common.BleachbitTestCase):
         self.assertFalse(is_junction(target_dir))
 
         from random import randint
-        canary_fn = os.path.join(target_dir, 'do_not_delete%d' % randint(1000,9999))
+        canary_fn = os.path.join(
+            target_dir, 'do_not_delete%d' % randint(1000, 9999))
         common.touch_file(canary_fn)
         self.assertExists(canary_fn)
         self.assertFalse(is_junction(canary_fn))
@@ -123,7 +124,8 @@ class WindowsTestCase(common.BleachbitTestCase):
 
         # create the link
         link_pathname = os.path.join(container_dir, 'link')
-        args = ('cmd', '/c', 'mklink', mklink_option, link_pathname, target_dir)
+        args = ('cmd', '/c', 'mklink', mklink_option,
+                link_pathname, target_dir)
         from bleachbit.General import run_external
         (rc, stdout, stderr) = run_external(args)
         self.assertEqual(rc, 0, stderr)
@@ -133,10 +135,6 @@ class WindowsTestCase(common.BleachbitTestCase):
         # put the link in the recycle bin
         move_to_recycle_bin(container_dir)
 
-        # preview the recycle bin
-        for f in get_recycle_bin():
-            print(f)
-
         def cleanup_dirs():
             shutil.rmtree(container_dir, True)
             self.assertNotExists(container_dir)
@@ -144,8 +142,6 @@ class WindowsTestCase(common.BleachbitTestCase):
 
         if not clear_recycle_bin:
             cleanup_dirs()
-            if not common.destructive_tests('windows link'):
-                self.skipTest('destructive tests are disabled')
             return
 
         # clear the recycle bin
@@ -170,13 +166,13 @@ class WindowsTestCase(common.BleachbitTestCase):
         """Unit test for directory symlink without clearing recycle bin"""
         self._test_link_helper('/d', False)
 
-    def test_link_junction_clear(self):
-        """Unit test for directory junctions with clearing recycle bin"""
-        self._test_link_helper('/j', True)
+    def test_link_symlink_clear(self):
+        """Unit test for directory symlink with clearing recycle bin"""
+        self._test_link_helper('/d', True)
 
     def test_delete_locked_file(self):
         """Unit test for delete_locked_file"""
-        tests = ('regular', u'unicode-emdash-u\u2014', 'long' + 'x' * 100)
+        tests = ('regular', 'unicode-emdash-u\u2014', 'long' + 'x' * 100)
         for test in tests:
             f = tempfile.NamedTemporaryFile(
                 prefix='bleachbit-delete-locked-file', suffix=test,
@@ -210,7 +206,7 @@ class WindowsTestCase(common.BleachbitTestCase):
         # create a nested key
         key = 'Software\\BleachBit\\DeleteThisKey'
         subkey = key + '\\AndThisKey'
-        hkey = _winreg.CreateKey(_winreg.HKEY_CURRENT_USER, subkey)
+        hkey = winreg.CreateKey(winreg.HKEY_CURRENT_USER, subkey)
         hkey.Close()
 
         # test
@@ -228,12 +224,12 @@ class WindowsTestCase(common.BleachbitTestCase):
         # UnicodeDecodeError: 'ascii' codec can't decode byte 0xc3 in position
         # 11: ordinal not in range(128)
         key = r'Software\\BleachBit\\DeleteThisKey'
-        hkey = _winreg.CreateKey(
-            _winreg.HKEY_CURRENT_USER, key + r'\\AndThisKey-Ö')
+        hkey = winreg.CreateKey(
+            winreg.HKEY_CURRENT_USER, key + r'\\AndThisKey-Ö')
         hkey.Close()
-        return_value = delete_registry_key(u'HKCU\\' + key, True)
+        return_value = delete_registry_key('HKCU\\' + key, True)
         self.assertTrue(return_value)
-        return_value = delete_registry_key(u'HKCU\\' + key, True)
+        return_value = delete_registry_key('HKCU\\' + key, True)
         self.assertFalse(return_value)
 
     def test_delete_registry_value(self):
@@ -245,11 +241,11 @@ class WindowsTestCase(common.BleachbitTestCase):
 
         # create a name-value pair
         key = 'Software\\BleachBit'
-        hkey = _winreg.CreateKey(_winreg.HKEY_CURRENT_USER, key)
+        hkey = winreg.CreateKey(winreg.HKEY_CURRENT_USER, key)
 
         value_name = 'delete_this_value_name'
-        _winreg.SetValueEx(
-            hkey, value_name, 0, _winreg.REG_SZ, 'delete this value')
+        winreg.SetValueEx(
+            hkey, value_name, 0, winreg.REG_SZ, 'delete this value')
         hkey.Close()
 
         # delete and confirm
@@ -276,8 +272,9 @@ class WindowsTestCase(common.BleachbitTestCase):
 
     def test_detect_registry_key(self):
         """Test for detect_registry_key()"""
-        self.assert_(detect_registry_key('HKCU\\Software\\Microsoft\\'))
-        self.assert_(not detect_registry_key('HKCU\\Software\\DoesNotExist'))
+        self.assertTrue(detect_registry_key('HKCU\\Software\\Microsoft\\'))
+        self.assertTrue(not detect_registry_key(
+            'HKCU\\Software\\DoesNotExist'))
 
     def test_get_clipboard_paths(self):
         """Unit test for get_clipboard_paths"""
@@ -295,7 +292,7 @@ class WindowsTestCase(common.BleachbitTestCase):
         win32clipboard.SetClipboardText(fname, win32clipboard.CF_TEXT)
         win32clipboard.SetClipboardText(fname, win32clipboard.CF_UNICODETEXT)
         self.assertEqual(win32clipboard.GetClipboardData(
-            win32clipboard.CF_TEXT), fname)
+            win32clipboard.CF_TEXT), fname.encode('ascii'))
         self.assertEqual(win32clipboard.GetClipboardData(
             win32clipboard.CF_UNICODETEXT), fname)
         win32clipboard.CloseClipboard()
@@ -315,15 +312,16 @@ class WindowsTestCase(common.BleachbitTestCase):
         for path in paths:
             self.assertExists(path)
 
+    def test_get_font_conf_file(self):
+        """Unit test for get_font_conf_file"""
+        # This tests only one of three situations.
+        font_fn = get_font_conf_file()
+        self.assertExists(font_fn)
+
     def test_get_known_folder_path(self):
         """Unit test for get_known_folder_path"""
         ret = get_known_folder_path('LocalAppDataLow')
         self.assertNotEqual(ret, '')
-        if parse_windows_build() <= 6.0:
-            # Before Vista
-            self.assertEqual(ret, None)
-            return
-        # Vista or later
         self.assertNotEqual(ret, None)
         self.assertExists(ret)
 
@@ -347,18 +345,20 @@ class WindowsTestCase(common.BleachbitTestCase):
         # check the function basically works
         for drive in get_fixed_drives():
             ret = empty_recycle_bin(drive, really_delete=False)
-            self.assertIsInstance(ret, (int, long))
-        if not common.destructive_tests('recycle bin'):
-            return
+            self.assertIsInteger(ret)
+
+    @common.skipUnlessDestructive
+    def test_empty_recycle_bin_destructive(self):
+        """Unit test the destructive part of empty_recycle_bin()"""
         # check it deletes files for fixed drives
         put_files_into_recycle_bin()
         for drive in get_fixed_drives():
             ret = empty_recycle_bin(drive, really_delete=True)
-            self.assertIsInstance(ret, (int, long))
+            self.assertIsInteger(ret)
         # check it deletes files for all drives
         put_files_into_recycle_bin()
         ret = empty_recycle_bin(None, really_delete=True)
-        self.assertIsInstance(ret, (int, long))
+        self.assertIsInteger(ret)
         # Repeat two for reasons.
         # 1. Trying to empty an empty recycling bin can cause
         #    a 'catastrophic failure' error (handled in the function)
@@ -373,11 +373,13 @@ class WindowsTestCase(common.BleachbitTestCase):
         There are more tests in testwipe.py
         """
 
-        from bleachbit.WindowsWipe import file_wipe
+        from bleachbit.WindowsWipe import file_wipe, open_file, close_file, file_make_sparse
+        from bleachbit.Windows import elevate_privileges
+        from win32con import GENERIC_WRITE, WRITE_DAC
 
         dirname = tempfile.mkdtemp(prefix='bleachbit-file-wipe')
 
-        filenames = ('short', 'long' + 'x' * 250, u'utf8-ɡælɪk')
+        filenames = ('short', 'long' + 'x' * 250, 'utf8-ɡælɪk')
         for filename in filenames:
             longname = os.path.join(dirname, filename)
             logger.debug('file_wipe(%s)', longname)
@@ -390,10 +392,30 @@ class WindowsTestCase(common.BleachbitTestCase):
                 self.assertExists(shortname)
                 return shortname
 
-            def _test_wipe(contents):
+            def _deny_access(fh):
+                import win32security
+                import ntsecuritycon as con
+
+                user, _, _ = win32security.LookupAccountName(
+                    "", win32api.GetUserName())
+                dacl = win32security.ACL()
+                dacl.AddAccessDeniedAce(
+                    win32security.ACL_REVISION, con.FILE_GENERIC_READ | con.FILE_GENERIC_WRITE, user)
+                win32security.SetSecurityInfo(fh, win32security.SE_FILE_OBJECT, win32security.DACL_SECURITY_INFORMATION,
+                                              None, None, dacl, None)
+
+            def _test_wipe(contents, deny_access=False, is_sparse=False):
                 shortname = _write_file(longname, contents)
-                logger.debug('test_file_wipe(): filename length={}, shortname length ={}, contents length={}'.format(
-                    len(longname), len(shortname), len(contents)))
+                if deny_access or is_sparse:
+                    fh = open_file(extended_path(longname),
+                                   mode=GENERIC_WRITE | WRITE_DAC)
+                    if is_sparse:
+                        file_make_sparse(fh)
+                    if deny_access:
+                        _deny_access(fh)
+                    close_file(fh)
+                logger.debug('test_file_wipe(): filename length={}, shortname length ={}, contents length={}, is_sparse={}'.format(
+                    len(longname), len(shortname), len(contents), is_sparse))
                 if shell.IsUserAnAdmin():
                     # wiping requires admin privileges
                     file_wipe(shortname)
@@ -407,10 +429,14 @@ class WindowsTestCase(common.BleachbitTestCase):
                 self.assertNotExists(shortname)
 
             # A small file that fits in MFT
-            _test_wipe('')
+            _test_wipe(b'')
 
             # requires wiping of extents
-            _test_wipe('secret' * 100000)
+            _test_wipe(b'secret' * 100000)
+
+            # requires wiping of extents: special file case
+            elevate_privileges(False)
+            _test_wipe(b'secret' * 100000, deny_access=True, is_sparse=True)
 
         shutil.rmtree(dirname, True)
 
@@ -437,17 +463,15 @@ class WindowsTestCase(common.BleachbitTestCase):
         """Unit test for setup_environment"""
         setup_environment()
         envs = ['commonappdata', 'documents', 'music', 'pictures', 'video',
-                'localappdata']
-        if parse_windows_build() >= 6.0:
-            envs.append('localappdatalow')
+                'localappdata', 'localappdatalow']
         for env in envs:
-            self.assertExists(os.environ[env].decode('utf8'))
+            self.assertExists(os.environ[env])
 
     def test_split_registry_key(self):
         """Unit test for split_registry_key"""
-        tests = (('HKCU\\Software', _winreg.HKEY_CURRENT_USER, 'Software'),
-                 ('HKLM\\SOFTWARE', _winreg.HKEY_LOCAL_MACHINE, 'SOFTWARE'),
-                 ('HKU\\.DEFAULT', _winreg.HKEY_USERS, '.DEFAULT'))
+        tests = (('HKCU\\Software', winreg.HKEY_CURRENT_USER, 'Software'),
+                 ('HKLM\\SOFTWARE', winreg.HKEY_LOCAL_MACHINE, 'SOFTWARE'),
+                 ('HKU\\.DEFAULT', winreg.HKEY_USERS, '.DEFAULT'))
         for (input_key, expected_hive, expected_key) in tests:
             (hive, key) = split_registry_key(input_key)
             self.assertEqual(expected_hive, hive)
@@ -479,12 +503,10 @@ class WindowsTestCase(common.BleachbitTestCase):
         self.assertEqual(ret, 0)
 
     def test_set_environ(self):
-        for folder in [u'folderäö', 'folder']:
+        for folder in ['folderäö', 'folder']:
             test_dir = os.path.join(self.tempdir, folder)
             os.mkdir(test_dir)
             self.assertExists(test_dir)
             set_environ('cd_test', test_dir)
-            if isinstance(test_dir, unicode):
-                test_dir = test_dir.encode('utf-8')
             self.assertEqual(os.environ['cd_test'], test_dir)
             os.environ.pop('cd_test')
